
----------------------------------------------------------------
----------------------------------------------------------------
---   VGA & CHAR ROM PACKAGE							--------
---														--------
----------------------------------------------------------------
----------------------------------------------------------------


LIBRARY IEEE;
USE  IEEE.STD_LOGIC_1164.all;
USE IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

PACKAGE p4pack IS
	COMPONENT vga_sync
 		PORT(clock_25Mhz, red, green, blue		: IN	STD_LOGIC;
         	 red_out, green_out, blue_out 		: OUT 	STD_LOGIC;
			 horiz_sync_out, vert_sync_out		: OUT 	STD_LOGIC;
			 pixel_row, pixel_column	: OUT STD_LOGIC_VECTOR(9 DOWNTO 0));
	END COMPONENT;
	COMPONENT char_rom
		PORT(character_address			: IN STD_LOGIC_VECTOR(5 DOWNTO 0);
			 font_row, font_col			: IN STD_LOGIC_VECTOR(2 DOWNTO 0);
			 rom_mux_output				: OUT	STD_LOGIC);
	END COMPONENT;
END p4pack;


----------------------------------------------------------------
----------------------------------------------------------------
---   Bouncing Ball Generation							--------
---														--------
----------------------------------------------------------------
----------------------------------------------------------------


LIBRARY IEEE;
USE  IEEE.STD_LOGIC_1164.all;
USE IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

LIBRARY work;
USE work.p4pack.all;
LIBRARY lpm;
USE lpm.lpm_components.ALL;

ENTITY BALL IS

   PORT( Clock 										: IN std_logic;             
         Red,Green,Blue 							: OUT std_logic;
         Horiz_sync,Vert_sync 						: OUT std_logic;
         R_sw,G_sw,B_sw 							: IN std_logic;
		 X_sw,Y_sw,Z_sw 							: IN std_logic;
		 speed		 								: IN std_logic_vector(1 DOWNTO 0)); 

END BALL;

ARCHITECTURE structure OF BALL IS

SIGNAL Vert_sync_in : STD_LOGIC;

-- Video Display Signals
SIGNAL Red_Data, Green_Data, Blue_Data, Power_On, Rev_video : std_logic;

-- Signals for Video ROM Memory for Pixel Data
SIGNAL char_address				: std_logic_vector(5 DOWNTO 0);
SIGNAL sum_address				: std_logic_vector(6 DOWNTO 0);
SIGNAL col_address, row_address	: std_logic_vector(5 DOWNTO 0);
SIGNAL pixel_col, pixel_row		: std_logic_vector(9 DOWNTO 0);
SIGNAL rom_mux_output			: std_logic;
SIGNAL format_address			: std_logic_vector(5 DOWNTO 0);
SIGNAL format_data				: std_logic_vector(5 DOWNTO 0);

SIGNAL ball_row					: std_logic_vector(4 DOWNTO 0) := "01010";
SIGNAL ball_col					: std_logic_vector(5 DOWNTO 0) := "010100";

SIGNAL direction_vert				: std_logic;
SIGNAL direction_horz				: std_logic;
SIGNAL shape						: std_logic_vector(2 DOWNTO 0);
BEGIN

 --Character Format ROM for Video Display -- Static
 format_rom: lpm_rom
      GENERIC MAP ( lpm_widthad => 6,
        lpm_numwords => 60,
        lpm_outdata => "UNREGISTERED",
        lpm_address_control => "UNREGISTERED",
			-- Reads in mif file for data display format
        lpm_file => "static.mif",
        lpm_width => 6)
      PORT MAP ( address => format_address, q => format_data);

				-- Generate VGA sync signals for display
   SYNC:  VGA_SYNC
 		PORT MAP(clock_25Mhz => clock, 
				red => red_data, green => green_data, blue => blue_data,	
    	     	red_out => red, green_out => green, blue_out => blue,
			 	horiz_sync_out => horiz_sync, vert_sync_out => vert_sync_in,
			 	pixel_row => pixel_row, pixel_column => pixel_col);
				-- Character Font ROM for Video Display
   CGROM: CHAR_ROM
		PORT MAP(character_address => char_address,
				font_row => pixel_row(3 DOWNTO 1), font_col => pixel_col(3 DOWNTO 1),	
				rom_mux_output => rom_mux_output);

Vert_sync <= vert_sync_in;

			-- Colors for pixel data on video signal
			-- address video_rom for pixel color data
			-- Switch 2 xor Rev_Video will reverse video
			-- Dip switchs change the color of the ball
			-- rom_mux_output is the data coming out the char rom
Red_Data <= not (rom_mux_output and R_sw);
Green_Data <= not (rom_mux_output and G_sw);
Blue_Data <= not (rom_mux_output and B_sw);

			-- current character row and column being displayed
row_address(5 DOWNTO 0) <= pixel_row(9 DOWNTO 4);
col_address(5 DOWNTO 0) <= pixel_col(9 DOWNTO 4);


		-- Address for Constant Character Data Format ROM
format_address(1 DOWNTO 0) <= Col_address(1 DOWNTO 0);
format_address(5 DOWNTO 2) <= Row_address(4 DOWNTO 1);

VIDEO_DISPLAY_DATA: PROCESS

variable cnt: integer range 0 to 25175000;
variable speed1: integer range 0 to 1000343 := 1000340;
BEGIN
  WAIT UNTIL (clock'event) AND (clock='1');


CASE speed IS
	WHEN "00" =>
	speed1 := 1000340;
	WHEN "01" =>
	speed1 := 1000340/2;
	WHEN "10" =>
	speed1 := 1000340/4;
	WHEN OTHERS =>
	speed1 := 1000340/8;
END CASE;

  IF(cnt = speed1+1) THEN
		cnt := 0;
  ELSE
		cnt := cnt + 1;
  END IF;

-- Reverse Video for Title at top of screen
--IF (row_address <= "00011") THEN rev_video <= '1'; 
--	ELSE rev_video <= '0';
--END IF;

						--ROW 30 GO BACK --DOWN--
  IF(ball_row = "11101") THEN
		direction_vert <= '1';
  END IF;
						--ROW 0 GO BACK --UP--
  IF(ball_row = "000100") THEN
		direction_vert <= '0';
  END IF;

						--ROW 30 GO BACK --DOWN--
  IF(ball_col = "100111") THEN
		direction_horz <= '1';
  END IF;
						--ROW 0 GO BACK --UP--
  IF(ball_col = "000000") THEN
		direction_horz <= '0';
  END IF;

					-----Move Ball Ever 1/2 Second----
  IF(cnt = speed1) THEN
	  IF(direction_vert = '0') THEN
			ball_row <= ball_row + 1;
	  END IF;
	  IF(direction_vert = '1') THEN
			ball_row <= ball_row - 1;	
	  END IF;	
	
	  IF(direction_horz = '0') THEN
			ball_col <= ball_col + 1;
	  END IF;
	  IF(direction_horz = '1') THEN
			ball_col <= ball_col - 1;	
	  END IF;		
  END IF;

shape(0) <= X_sw;
shape(1) <= Y_sw;
shape(2) <= Z_sw;
								--THE BALL!
IF (row_address = ball_row) AND (col_address = ball_col) THEN
		
		CASE shape IS
			WHEN "000" =>
				char_address <= "001111";
			WHEN "001" =>
				char_address <= "110000";
			WHEN "010" =>
				char_address <= "110001";
			WHEN "011" =>		
				char_address <= "010011";
			WHEN "100" =>		
				char_address <= "110010";
			WHEN "101" =>	
				char_address <= "110011";
			WHEN "110" =>
				char_address <= "011110";
			WHEN OTHERS =>
				char_address <= "110100";					
		END CASE;
								--The
ELSIF (row_address = "00010") AND (col_address = "00111") THEN
		char_address <= "010100";
ELSIF (row_address = "00010") AND (col_address = "01000") THEN
		char_address <= "001000";		
ELSIF (row_address = "00010") AND (col_address = "01001") THEN
		char_address <= "000101";		
								--Space
ELSIF (row_address = "00010") AND (col_address = "01010") THEN
		char_address <= "100000";
								--Amazing
ELSIF (row_address = "00010") AND (col_address = "01011") THEN
		char_address <= "000001";				
ELSIF (row_address = "00010") AND (col_address = "01100") THEN
		char_address <= "001101";		
ELSIF (row_address = "00010") AND (col_address = "01101") THEN
		char_address <= "000001";
ELSIF (row_address = "00010") AND (col_address = "01110") THEN
		char_address <= "011010";
ELSIF (row_address = "00010") AND (col_address = "01111") THEN
		char_address <= "001001";
ELSIF (row_address = "00010") AND (col_address = "10000") THEN
		char_address <= "001110";
ELSIF (row_address = "00010") AND (col_address = "10001") THEN
		char_address <= "000111";
								--Space
ELSIF (row_address = "00010") AND (col_address = "10010") THEN
		char_address <= "100000";
			
								--Bouncing
ELSIF (row_address = "00010") AND (col_address = "10011") THEN
		char_address <= "000010";		
ELSIF (row_address = "00010") AND (col_address = "10100") THEN
		char_address <= "001111";				
ELSIF (row_address = "00010") AND (col_address = "10101") THEN
		char_address <= "010101";		
ELSIF (row_address = "00010") AND (col_address = "10110") THEN
		char_address <= "001110";
ELSIF (row_address = "00010") AND (col_address = "10111") THEN
		char_address <= "000011";
ELSIF (row_address = "00010") AND (col_address = "11000") THEN
		char_address <= "001001";
ELSIF (row_address = "00010") AND (col_address = "11001") THEN
		char_address <= "001110";
ELSIF (row_address = "00010") AND (col_address = "11010") THEN
		char_address <= "000111";
		
								--Space
ELSIF (row_address = "00010") AND (col_address = "11011") THEN
		char_address <= "100000";		
		
								--Ball
ELSIF (row_address = "00010") AND (col_address = "11100") THEN
		char_address <= "000010";
ELSIF (row_address = "00010") AND (col_address = "11101") THEN
		char_address <= "000001";			
ELSIF (row_address = "00010") AND (col_address = "11110") THEN
		char_address <= "001100";		
ELSIF (row_address = "00010") AND (col_address = "11111") THEN
		char_address <= "001100";		
ELSIF (row_address = "00011") THEN
		char_address <= "101101";
ELSE
		char_address <= "100000";
END IF;

END PROCESS VIDEO_DISPLAY_DATA;

END structure;


----------------------------------------------------------------
----------------------------------------------------------------
---   VGA SYNC GENERATION								--------
---														--------
----------------------------------------------------------------
----------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.all;
USE IEEE.STD_LOGIC_ARITH.all;
USE IEEE.STD_LOGIC_UNSIGNED.all;

-- VGA Video Sync generation

ENTITY VGA_SYNC IS

	PORT(clock_25Mhz, red, green, blue		: IN	STD_LOGIC;
		 red_out, green_out, blue_out, 
		 horiz_sync_out, vert_sync_out		: OUT	STD_LOGIC;
		 pixel_row, pixel_column			: OUT STD_LOGIC_VECTOR(9 DOWNTO 0));

END VGA_SYNC;

ARCHITECTURE a OF VGA_SYNC IS
	SIGNAL horiz_sync, vert_sync 			: STD_LOGIC;
	SIGNAL video_on, video_on_v, video_on_h : STD_LOGIC;
	SIGNAL h_count, v_count 				: STD_LOGIC_VECTOR(9 DOWNTO 0);

BEGIN

			-- video_on is high only when RGB data is being displayed
 video_on <= video_on_H AND video_on_V;

--Generate Horizontal and Vertical Timing Signals for Video Signal

 PROCESS
 BEGIN
	WAIT UNTIL(clock_25Mhz'EVENT) AND (clock_25Mhz='1');

	-- H_count counts pixels (640 + extra time for sync signals)
	-- 
	--  Horiz_sync  ------------------------------------__________--------
	--  H_count       0                640             659       755    799
	--
	IF (h_count = 799) THEN
   		h_count <= "0000000000";
	ELSE
   		h_count <= h_count + 1;
	END IF;

	--Generate Horizontal Sync Signal using H_count
	IF (h_count <= 755) AND (h_count >= 659) THEN
 	  	horiz_sync <= '0';
	ELSE
 	  	horiz_sync <= '1';
	END IF;


	--V_count counts rows of pixels (480 + extra time for sync signals)
	--  
	--  Vert_sync      ----------------------------------_______------------
	--  V_count         0                         480    493-494          524
	--
	IF (v_count >= 524) AND (h_count >= 699) THEN
   		v_count <= "0000000000";
	ELSIF (h_count = 699) THEN
   		v_count <= v_count + 1;
	END IF;

		-- Generate Vertical Sync Signal using V_count
	IF (v_count <= 494) AND (v_count >= 493) THEN
   		vert_sync <= '0';
	ELSE
  		vert_sync <= '1';
	END IF;

		-- Generate Video on Screen Signals for Pixel Data
	IF (h_count <= 639) THEN
   		video_on_h <= '1';
   		pixel_column <= h_count;
	ELSE
	   	video_on_h <= '0';
	END IF;

	IF (v_count <= 479) THEN
   		video_on_v <= '1';
   		pixel_row <= v_count;
	ELSE
   		video_on_v <= '0';
	END IF;

				-- Put all video signals through DFFs to elminate 
				-- any logic delays that can cause a blurry image
		red_out 		<= red AND video_on;
		green_out 		<= green AND video_on;
		blue_out 		<= blue AND video_on;
		horiz_sync_out 	<= horiz_sync;
		vert_sync_out 	<= vert_sync;

 END PROCESS;

END a;



----------------------------------------------------------------
----------------------------------------------------------------
---   Character GENERATION	Rom							--------
---														--------
----------------------------------------------------------------
----------------------------------------------------------------



LIBRARY IEEE;
USE  IEEE.STD_LOGIC_1164.all;
USE IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;
LIBRARY lpm;
USE lpm.lpm_components.ALL;

ENTITY Char_ROM IS
	PORT(	character_address		: IN	STD_LOGIC_VECTOR(5 DOWNTO 0);
			font_row, font_col		: IN 	STD_LOGIC_VECTOR(2 DOWNTO 0);
			rom_mux_output			: OUT	STD_LOGIC);
END Char_ROM;

ARCHITECTURE struct OF Char_ROM IS
	SIGNAL	rom_data			: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL	rom_address			: STD_LOGIC_VECTOR(8 DOWNTO 0);
BEGIN
				-- Small 8 by 8 Character Generator ROM for Video Display
				-- Each character is eight 8-bits words of pixel data
 char_gen_rom: lpm_rom
      GENERIC MAP ( lpm_widthad => 9,
        lpm_numwords => 512,
        lpm_outdata => "UNREGISTERED",
        lpm_address_control => "UNREGISTERED",
				-- Reads in mif file for character generator font data 
        lpm_file => "char_set.mif",
        lpm_width => 8)
      PORT MAP ( address => rom_address, q => rom_data);

rom_address <= character_address & font_row;

				-- Mux to pick off correct rom data bit from 8-bit word
				-- for on screen character generation
rom_mux_output <= rom_data ( (CONV_INTEGER(NOT font_col(2 DOWNTO 0))));

END struct;